library(yuima)

### Homogeneous Poisson Process ###

# Construction of a Poisson Process.
mod1 <- setPoisson(intensity = "alpha", df = list("dconst(z, 1)"))
class(mod1) # yuima.poisson-class

# Simulation
set.seed(1)
samp <- setSampling(0, 70, n = 7000)
system.time(
poisson1 <- simulate(mod1, sampling = samp, true.parameter = list(alpha = 1))
)
# plot

plot(poisson1,ylab=expression(N[t]))


# Estimation of a Poisson Process 

res1pois <- qmle(poisson1, start = list(alpha=1), 
                 method = "L-BFGS-B", lower=list(alpha=0))

class(res1pois) # yuima.CP.qmle
summary(res1pois)


### Compound Poisson Process ###

mod2 <- setPoisson(intensity = "alpha", df = list("dnorm(z, 0, 1)"))
class(mod2)

set.seed(1)
poisson2 <- simulate(mod2, sampling = samp, true.parameter = list(alpha = 1))

plot(poisson2)


### Inhomogeneous Poisson Process ###

# Inhomogeneous Poisson Process
mod2 <- setPoisson(intensity = "alpha+beta*cos(2*t*3.14/zeta)", df = list("dconst(z, 1)"))
set.seed(1)

poisson2 <- simulate(mod2, sampling = samp, true.parameter = list(alpha = 1, beta = 1/2, zeta = 23))

plot(poisson2)

res2pois <- qmle(poisson2, start = list(alpha = 1, beta = 1/2, zeta = 23), 
                 method = "L-BFGS-B", lower = list(alpha = 0, beta = 0, zeta = 0.1))

summary(res2pois)
### Re-estimate Poisson Process with hawkes###
### Hawkes Process ###
# Law definition
# Def of rng
my.rPpr1<-function(n,t){
  res<-matrix(1,n,1)
  return(res)
}
#Def of dens
my.dPpr1<-function(x, t){
  res<-1
  return(res)
}
#setLaw: constructor of an obj of class yuima.law
myLaw<- setLaw(rng = my.rPpr1, density = my.dPpr1)

myCov <- setModel(drift = c("0"), diffusion = c("0"),
                  jump.coeff = matrix(c("1"), 1, 1), measure = list(df = myLaw),
                  measure.type = c("code"), solve.variable = c("N"),
                  state.variable = c("N"), xinit = c("0"))

yuimaPPR1nested <- setPPR(yuima = myCov, gFun = "alpha",
         Kernel = as.matrix("beta*exp(-(t-s))"), var.dx = "N")

class(yuimaPPR1nested)

originaldata1 <- poisson1@data@original.data
colnames(originaldata1) <- "N"
myPPRdata1n <- DataPPR(originaldata1, yuimaPPR1nested, samp)

# Estimation of a PPR model
res1PPRn <- qmle(myPPRdata1n, start = list(alpha = 1, beta = 1), method = "L-BFGS-B",
   lower = list(alpha = 0, beta = 0))
# Comparison results
# yuima.CP.qmle
coef(res1pois) # Estimates
sqrt(diag(res1pois@vcov)) # Standard Error
coef(res1PPRn) # yuima.PPR.qmle
sqrt(diag(res1PPRn@vcov)) # Standard Error


### Re-estimate Poisson Process with PPR algorithms ###

yuimaPPR1 <- setPPR(yuima = myCov, gFun = "alpha", 
                    Kernel = as.matrix("0"), var.dx = "N")
myPPRdata1 <- DataPPR(originaldata1, yuimaPPR1, samp)

res1PPR <- qmle(myPPRdata1, start = list(alpha = 1),
                method = "L-BFGS-B", lower = list(alpha = 0))

# Results
coef(res1pois) 
sqrt(diag(res1pois@vcov)) 
coef(res1PPR)  
sqrt(diag(res1PPR@vcov)) 

### Re-estimate Inhomogeneous Poisson Process ###
#### alpha+beta*cos(2*t*3.14/zeta) ####

originaldata2 <- poisson2@data@original.data

# Point Process Regression Model without kernel
yuimaPPR2 <- setPPR(yuima = myCov, gFun = "alpha+beta*cos(2*t*3.14/zeta)",
                    Kernel = as.matrix("0"),var.dx = "N")
colnames(originaldata2) <- "N"
myPPRdata2 <- DataPPR(originaldata2, yuimaPPR2, samp)

res2PPR <- qmle(myPPRdata2, start = list(alpha = 1,beta = 1/2, zeta = 23),
                method = "L-BFGS-B", lower = list(alpha = 0, beta = 0, zeta = 0.1))

# Point Process Regression Model with exponential kernel
yuimaPPR2Nested <- setPPR(yuima = myCov, gFun = "alpha+beta*cos(2*t*3.14/zeta)", 
                          Kernel = as.matrix("gamma*exp(-(t-s))"), var.dx = "N")
myPPRdata2n <- DataPPR(originaldata2, yuimaPPR2Nested, samp)

res2PPRn <- qmle(myPPRdata2n, start = list(alpha = 1, beta = 1/2, zeta = 23, gamma = 0),
                 method = "L-BFGS-B", lower = list(alpha = 0, beta = 0, zeta =.1, gamma = 0))

# Results
coef(res2pois)
sqrt(diag(res2pois@vcov))
coef(res2PPR)
sqrt(diag(res2PPR@vcov))
coef(res2PPRn)
sqrt(diag(res2PPRn@vcov))

## Simulate Poisson process using Hawkes ##
set.seed(1)
system.time(
Haw1 <- simulate(yuimaPPR1nested,
  true.parameter = list(alpha=1,beta=0),sampling = samp)
)

plot(Haw1)


### Univariate Simultaneous Evolution Model with Intensity in the Dynamics of the Covariates

# Law definition
my.rFITC1 <- function(n,t){
  res0 <- as.matrix(rgamma(n, 0.1*t, rate = sqrt(0.1))-sqrt(0.1)*t)
  res1 <- as.matrix(rep(1,n))
  res <- cbind(res0,res1)
  return(res)
}
Law.FITC1 <- setLaw(rng = my.rFITC1)

# Point Process Regression Model definition
modFITC1 <- setModel(drift =c("mu*(0.5-X)+b*lamb+0.1*vol","0"), 
                     diffusion = c("0","0"), jump.coeff = matrix(c("vol*sqrt(0.1)","0","0","1"),2,2),
                     measure = list(df = Law.FITC1), measure.type = c("code","code"), 
                     solve.variable = c("X","N"), state.variable = c("X","N","lamb"), 
                     xinit=c("0.25","0"))

# object of yuima.PPR-class
g.FITC1 <- "exp(mu*log(1+X))"
Kern.FITC1 <- "alpha*exp(-beta*(t-s))"
CovFITC1 <- setPPR(yuima = modFITC1, counting.var = "N", gFun = g.FITC1,
                   Kernel = as.matrix(Kern.FITC1), var.dx = "N", lambda.var = "lamb")

parFITC1 <- list(b = 0.21, vol = 1, mu = 0.35, 
                 alpha = 3.5, beta = 4.5)

Term<-70
n<-7000
set.seed(1)
system.time(
simFITC1 <- simulate(object = CovFITC1, true.parameter = parFITC1,
                     sampling = setSampling(Terminal =Term, n=n))
)

plot(simFITC1, main = " " , ylab = expression(X[t], N[t]))

system.time(
resFITC1 <- qmle(simFITC1, start = parFITC1)
)

summary(resFITC1$PPR)

resFITC1$Covariates
